@@ -3,123 +3,65 @@ from selenium.webdriver.common.action_chains import ActionChains
 from selenium.webdriver.common.utils import keys_to_typing
 from selenium.webdriver.common.keys import Keys
 from selenium.common.exceptions import WebDriverException
-from . import useless_logger
+
+from .pageobjectbase import PageObjectBase
 
 
-class PageObject(object):
+class PageObject(PageObjectBase):
 
-    def __init__(self, locator, parent, chain=True, webdriver=None, logger=useless_logger, name=None, children_class=None):
+    DEFAULT_WAIT_TIMEOUT = 60
+
+
+    def __init__(self, locator, parent, chain=True, webdriver=None, logger=None, name=None):
         self._locator = locator
         self.parent = parent
         self._chain = chain
         self._webdriver = webdriver
         self._logger = logger
         self._name = name
-        self._children_class = children_class
+
+        try:
+            self.parent.register_child(self)
+        except AttributeError:
+            pass
 
         self.init_children()
 
 
+    def __bool__(self):
+        return self.is_existing(log=False)
+
+
+    def __getitem__(self, key):
+        return self.children[key]
+
+
+    def __len__(self):
+        return len(self.children)
+
+
+    def register_child(self, child):
+        try:
+            self.__setattr__(child.name, child)
+        except TypeError:
+            return
+
+
     def init_children(self):
-        '''
+        """
         Meant to be overloaded by page objects
         containing other page objects.
-        '''
+        """
         pass
 
 
-    @property
-    def locator(self):
-        try:
-            if self._chain:
-                return self.parent.locator + self._locator
-            else:
-                return self._locator
-        except AttributeError:
-            return self._locator
-
-
-    @property
-    def webdriver(self):
-        try:
-            return self.parent.webdriver
-        except AttributeError:
-            return self._webdriver
-
-
-    @property
-    def logger(self):
-        try:
-            return self.parent.logger
-        except AttributeError:
-            return self._logger
-
-
     @property
     def children(self):
-        return {attr_name for attr_name, attr_value in self.__dict__.items()
-                if isinstance(attr_value, PageObject)
+        return {attr_name: attr_value for attr_name, attr_value in self.__dict__.items()
+                if isinstance(attr_value, PageObjectBase)
                 and attr_value is not self.parent}
 
 
-    @property
-    def name(self):
-        if self._name:
-            return self._name
-        try:
-            if isinstance(self.parent.children, list):
-                return '{}[{}]'.format(self.parent.name, str(self.index))
-            for child in self.parent.children:
-                if self.parent.__dict__[child] == self:
-                    return child
-        except AttributeError:
-            return 'root'
-        except KeyError:
-            return 'page_object'
-
-
-    @property
-    def full_name(self):
-        try:
-            if isinstance(self.parent.children, set):
-                return '{}.{}'.format(self.parent.full_name, self.name)
-            else:
-                return '{}[{}]'.format(self.parent.full_name, str(self.index))
-        except AttributeError:
-            return self.name
-
-
-    @property
-    def short_full_name(self):
-        if self.name == self.full_name:
-            return self.name
-        else:
-            return '...{}'.format(self.name)
-
-
-    @property
-    def children_class(self):
-        raise NotImplementedError
-
-
-    @property
-    def _log_id_short(self):
-        return 'page object "{}"'.format(self.name)
-
-
-    @property
-    def _log_id_long(self):
-        return 'full name path: "{}", element: "{}"'.format(
-                self.full_name, self.locator)
-
-
-    def __repr__(self):
-        my_class = self.__class__.__name__
-        base_class = self.__class__.__bases__[0].__name__
-        return '<{}({}) (locator="{}")>'.format(
-                my_class, base_class, self.locator)
-
-
     def find(self, log=True):
         if log:
             self.logger.info('looking for {}'.format(self._log_id_short))
@@ -166,31 +108,31 @@ class PageObject(object):
 
 
 
-    def wait_for_exist(self, timeout=60):
+    def wait_for_exist(self, timeout=DEFAULT_WAIT_TIMEOUT):
         self.logger.info('waiting until page contains {}'.format(self._log_id_short))
         self.logger.debug('waiting until page contains page object; {}'.format(self._log_id_long))
         WebDriverWait(self.webdriver, timeout).until(
                 lambda x: self.is_existing(log=False),
-                message='Element "{}" not found after {} seconds'.format(
+                message='Element "{}" still not existing after {} seconds'.format(
                     self.locator, timeout))
         self.logger.info('finished waiting until page contains {}'.format(self._log_id_short))
         self.logger.debug('finished waiting until page contains page object; {}'.format(self._log_id_long))
         return self
 
 
-    def wait_for_vanish(self, timeout=60):
+    def wait_for_vanish(self, timeout=DEFAULT_WAIT_TIMEOUT):
         self.logger.info('waiting until page does not contain {}'.format(self._log_id_short))
         self.logger.debug('waiting until page does not contain page object; {}'.format(self._log_id_long))
         WebDriverWait(self.webdriver, timeout).until(
                 lambda x: not self.is_existing(log=False),
-                message='Element "{}" not found after {} seconds'.format(
+                message='Element "{}" still existing after {} seconds'.format(
                     self.locator, timeout))
         self.logger.info('finished waiting until page does not contain {}'.format(self._log_id_short))
         self.logger.debug('finished waiting until page does not contain page object; {}'.format(self._log_id_long))
         return self
 
 
-    def wait_for_visible(self, timeout=60):
+    def wait_for_visible(self, timeout=DEFAULT_WAIT_TIMEOUT):
         self.logger.info('waiting until {} is visible'.format(self._log_id_short))
         self.logger.debug('waiting until page object is visible; {}'.format(self._log_id_long))
         WebDriverWait(self.webdriver, timeout).until(
